#include "dbase.h"

/* USAGE:

   ADDFIELD <database> FIELDNAME TYPE'

   NOTE: The only valuable change above this line is that the InitList() must
   be performed in the template() function even if it can't read the database
   file!

   */

int addfield( int argc, char* argv[] )
{

  char dbname[256];
  FILE *file;
  t_fieldTemplate* template;
  t_fieldType fieldtype;
  char *type;
  char *name;

  if(argc < 5) {
    fprintf(stderr, "Insufficiant amount of arguments!\n");
    return -1;
  }
  name=argv[3];
  type=argv[4];
  /* check for valid types */
  switch(tolower(type[0])) {
  case 'b':
    fieldtype = binary;
    type="binary";
    break;
  case 's':
    fieldtype = string;
    type="string";
    break;
  case 'i':
    fieldtype = integer;
    type="integer";
    break;
  default:
    fprintf(stderr, "Unknown field type '%s'!\n", type );
    return -1;
  }

  /* create .db filename (Daniel's note: should be a function) */
  sprintf( dbname, "%s%s", argv[2], DB_TEMPLATE_EXTENSION );

  /* read database */
  if (readtemplate( argv[2] )) {
    /* no database found, create it! */

    time_t now = time(NULL);

    file = fopen( dbname, "w" );
    if(!file) {
      fprintf(stderr, "Can't create database template %s!\n", dbname);
      exit(1);
    }
    fprintf(file,
            "###########################################################################\n"
            "# Name of database: %s\n"
            "# Date of creation: %s"
            "#\n"
            "# The format of each line is like this:\n"
            "#  [field name]:[type]\n"
            "#\n"
            "# The field name may only contain characters [A-Za-z0-9]\n"
            "# The type is one of these three:\n"
            "#  * int    - standard 32/64-bit integer\n"
            "#  * string - character string, where NL is the only control character allowed\n"
            "#  * bin    - binary data - anything goes\n"
            "#\n"
            "# Lines beginning with # or whitespace are ignored\n"
            "#\n",
            argv[2], ctime(&now));
    fclose(file);
  }

  /* check for the duplice field name entries, and then add it */
  
  for ( template = (t_fieldTemplate*) GetFirst( &dbTemplate ); 
        template;
        template = (t_fieldTemplate*) GetNext( &template->node ) ) {
    
    if ( !strcmp( name, template->name ) )
      break;
  }
  if( template ) {
    fprintf(stderr, "Field name '%s' already in use, try another!\n",
            name);
    exit(1);
  }
  file = fopen( dbname, "a" );
  if(!file) {
    fprintf(stderr, "Can't open the database template %s!\n", dbname);
    exit(1);
  }
  fprintf(file, "%s:%s\n", name, type);
  fclose(file); 

  return 0;
}
