#include "dbase.h"

/* add a row to database */
/* params: 1: 'add' 2:filename 3:field 4:data/@filename [...] */
int addrow( int argc, char* argv[] )
{
    FILE* file;
    char dbname[256];
    int i,donefields=0;
    t_field* field;
    t_fieldTemplate* fieldTemplate;
    t_row row;
    t_textType textMode = text;
    int startarg=3;

    if ( argc < 3 ) {
        printf("usage: %s add [db] {-u | -h} [field id #1] [data #1/@file1] ...\n", argv[0]);
        return -1;
    }

    /* read database template */
    if (readtemplate( argv[2] )) {
        perror(argv[2]);
        return -1;
    }

    /* open data file */
    sprintf( dbname, "%s%s", argv[2], DB_FILE_EXTENSION );
    file = fopen( dbname, "a" );
    if (!file) {
        perror( dbname );
        return -1;
    }

    InitList( &row.fieldList, NO_SIZE_CHECK );

    if ( argv[3][0] == '-' ) {
        startarg++;
        if ( argv[3][1] == 'u' )
            textMode=url;
    }

    if ( (argc - startarg) & 1 ) {
        printf("field id / field data mismatch\n");
        return -1;
    }
    
    /* insert data in fields */
    for ( i=startarg; i<argc; i++ ) {
        int size;
        unsigned char* data;

        for ( fieldTemplate = (t_fieldTemplate*) GetFirst( &dbTemplate ); 
              fieldTemplate;
              fieldTemplate = (t_fieldTemplate*) GetNext( &fieldTemplate->node ) ) {
            if ( strcmp( argv[i], fieldTemplate->name ) )
                continue;

            /* create actual field */
            field = (t_field*) malloc( sizeof( t_field ) );
            field->template = fieldTemplate;

            /* is data in a file? */
            if ( argv[i+1][0] == '@' ) {
                FILE* infile;

                size=filesize( argv[i+1]+1 );
                data=(unsigned char*) malloc( size );

                /* read file */
                infile=fopen(argv[i+1]+1, "rb");
                if (!infile) {
                    perror(argv[i+1]);
                    fclose(file);
                    return -1;
                }
                fread( data, size, 1, infile );
                fclose( infile );
            }
            else {

                /* if field is integer, store as integer */
                if ( fieldTemplate->type == integer ) {
                    char* p1 = argv[i+1];
                    char* p2;
                    field->value = strtol( p1, &p2, 10 );
                    if ( p1 == p2 ) {
                        printf("Field '%s' is an integer.\n", fieldTemplate->name );
                        return -1;
                    }
                }
                else {
                    if ( url == textMode ) {
                        data=url2string( argv[i+1] );
                        size=strlen( data );
                    }
                    else {
                        size=strlen( argv[i+1] );
                        data=(unsigned char*) malloc( size + 1 );
                        strcpy( data, argv[i+1] );
                    }
                }
            }

            field->len   = size;
            field->data  = data;

            AddLast( &row.fieldList, &field->node );
            donefields++;
            break;
        }

        if ( !fieldTemplate ) {
            printf("Database '%s' has not defined a field called '%s'!\n",
                   argv[2], argv[i] );
            return -1;
        }

        i++;
    }

#if 0
    if ( donefields < dbTemplate.num_nodes ) {
        printf("You must supply data for ALL fields!\n");
        return -1;
    }
#endif

    /* create row id */
    {
        struct timeval tp;

        gettimeofday( &tp, NULL );

        /* output new-row marker */
        row.id1 = tp.tv_sec;
        row.id2 = tp.tv_usec;
    }

    /* write row to file */
    writerow( file, dbname, &row );

    fclose( file );

    return 0;
}
