#include "dbase.h"

/* modify a field in database */
int modify( int argc, char* argv[] )
{
    char dbname[256], bakname[128];
    FILE* file;
    t_row* row;
    t_field* field;
    unsigned long id1, id2;
    int size, i, startarg=4;
    unsigned char* data;
    char* rowarg;
    t_textType textMode=text;

    if ( argc < 5 ) {
        printf("usage: %s modify [database] [row id] {-u} [field #1] [data #1/@file #1] ...\n", argv[0]);
        return -1;
    }

    if ( argv[4][0] == '-' ) {
        startarg++;
        if ( argv[4][1] == 'u' )
            textMode=url;
    }

    if ( (argc - startarg) & 1 ) {
        printf("field id / field data mismatch\n");
        return -1;
    }

    rowarg = argv[3];

    if ( 2 != sscanf( rowarg, "%08lx%08lx", &id1, &id2 )) {
        printf("Row ID must be the 16 character row id string. '%s' is invalid.\n", rowarg);
        return -1;
    }

    /* read database */
    if (readdb( argv[2] )) {
        perror(argv[2]);
        return -1;
    }

    /* find row */
    for ( row = (t_row*) GetFirst( &rowList ); row;
          row = (t_row*) GetNext( &row->node ) ) {
        if ( (row->id1 == id1) && (row->id2 == id2) )
            break;
    }
    if (!row) {
        printf("Row ID '%s' not found.\n", rowarg );
        return -1;
    }

    /* go through the parameters */
    for ( i=startarg; i<argc; i+=2 ) {
        char* fieldarg = argv[i];
        char* dataarg = argv[i+1];

        /* find field */
        for ( field = (t_field*) GetFirst( &row->fieldList ); field;
              field = (t_field*) GetNext( &field->node ) )
            if ( !strcmp( field->template->name, fieldarg ) )
                break;

        /* row does not contain field? */
        if ( !field ) {
            t_fieldTemplate* template;

            /* is field defined in template? */
            for ( template = (t_fieldTemplate*) GetFirst( &dbTemplate ); 
                  template;
                  template = (t_fieldTemplate*) GetNext( &template->node ) ) {
                if (!strcmp( template->name, fieldarg ))
                    break;
            }
            
            if (!template) {
                printf("Database '%s' has not defined a '%s' field.\n",
                       argv[2], fieldarg );
                return -1;
            }
            else {
                /* create new field */
                field = (t_field*) malloc( sizeof( t_field ) );
                field->template = template;
				field->data = NULL;
				field->len = 0;
                AddLast( &row->fieldList, &field->node );
            }
        }
    
        /* is data in a file? */
        if ( dataarg[0] == '@' ) {
            FILE* infile;

            size=filesize( dataarg+1 );
            data=(unsigned char*) malloc( size );
            
            /* read file */
            infile=fopen(dataarg+1, "rb");
            if (!infile) {
                perror(dataarg);
                fclose(file);
                return -1;
            }
            fread( data, size, 1, infile );
            fclose( infile );
        }
        else {
            
            /* if field is integer, store as integer */
            if ( field->template->type == integer ) {
                char* p1 = dataarg;
                char* p2;
                field->value = strtol( p1, &p2, 10 );
                if ( p1 == p2 ) {
                    printf("Field '%s' is an integer.\n", field->template->name );
                    return -1;
                }
            }
            else {
                if ( url == textMode ) {
                    data=url2string( dataarg );
                    size=strlen(data);
                }
                else {
                    size=strlen( dataarg );
                    data=(unsigned char*) malloc( size + 1 );
                    strcpy( data, dataarg );
                }
            }
        }

        /* replace old data with new */
        if ( field->data )
            free( field->data );
        field->data = data;
        field->len  = size;
    }


    /* backup old file */
    sprintf( dbname, "%s" DB_FILE_EXTENSION, argv[2] );
    sprintf( bakname, "%s" DB_FILE_EXTENSION DB_BAK_EXTENSION, argv[2] );
    remove( bakname );
    rename( dbname, bakname );

    /* create new file */
    file = fopen( dbname, "w" );
    if (!file) {
        perror( dbname );
        return -1;
    }

    /* write all rows */
    for ( row = (t_row*) GetFirst( &rowList ); row;
          row = (t_row*) GetNext( &row->node ) )
        if (writerow( file, dbname, row ))
            return -1;

    return 0;
}
